#!/usr/bin/env python3

"""Initialize logger and hand off control to the controller module"""

import logging
import os
import sys
from datetime import datetime

from soutools import controller, helpers, settings

helpers.clear_screen()


def main():
    get_settings = settings.Settings()

    # Get logging information from settings
    file_log_level = get_settings.get_value("logging.file_log_level")
    file_log_path = get_settings.get_value("logging.file_log_path")

    # Setup log path and ensure that folders exist. If not, then create them
    home_dir = os.path.expanduser("~")
    log_path = os.path.join(home_dir, file_log_path)
    helpers.setup_folder_structure(log_path)
    log_file = f"{log_path}soutools_log__{datetime.now():%Y-%m-%d_%H-%M-%S}.log"

    # Setup a basic logger
    logger = logging.getLogger("soutools")
    logger.setLevel(logging.DEBUG)

    # Setup file handler and add it to the logger
    fh = logging.FileHandler(filename=log_file)
    # Validate log level is a proper instance
    file_level = getattr(logging, file_log_level.upper(), None)
    if isinstance(file_level, int):
        fh.setLevel(level=file_level)
    else:
        error = f'Invalid value "{file_log_level}" ' "specified for file_log_level"
        error = helpers.colorme(error, "red")
        blue_text = helpers.colorme("file_log_level", "blue")
        error += f"\n  Check {blue_text} in the settings.toml"
        error += (
            '\n  Valid values are "debug", "info", "warning", '
            '"error", and "critical"'
        )
        print()
        sys.exit("  " + error)
    # create formatter and add it to the handler
    formatter = logging.Formatter(
        "%(asctime)2s - %(name)19s - %(levelname)8s - %(message)s"
    )
    fh.setFormatter(formatter)
    # add the handler to the logger
    logger.addHandler(fh)

    # create console handler and add it to the logger
    ch = logging.StreamHandler()
    ch.setLevel(level="WARNING")
    # create formatter and add it to the handler
    formatter = logging.Formatter("%(levelname)8s - %(message)s")
    ch.setFormatter(formatter)
    # add the handlers to the logger
    logger.addHandler(ch)

    logger.debug("Logger initialized, calling controller menu function")
    controller.main_menu()


if __name__ == "__main__":
    main()



# menu.py

"""This is used to handle the menu system"""

from collections import namedtuple

from soutools import helpers

Option = namedtuple("Option", ["label", "callback"])


class Menu:
    def __init__(self, title, options):
        self.title = title
        self._options = [Option(*option) for option in options]

    def display(self):
        helpers.clear_screen()
        print(self.title)
        menu_item = ""
        for i, option in enumerate(self._options, start=1):
            menu_item += helpers.colorme(f"    {str(i)} - {option.label}\n", "green")
        print(menu_item)

    def callback(self, i):
        if i <= len(self._options):
            return self._options[i - 1].callback

    def get_input(self):
        while True:
            try:
                self.display()
                selection = int(input("\n  Select an option >> "))
                if selection not in range(1, len(self._options) + 1):
                    print("\nEntry not in range, please try again: ")
                    input("Press Enter to Continue\n")
                    self.display()
                    continue
                else:
                    self.display()
                    return self._options[selection - 1].callback()
            except ValueError:
                print("\nEntry needs to be an integer, please try again: ")
                input("Press Enter to Continue\n")
                self.display()
                continue



# settings.py

"""Used to create settings file if not exists and get settings"""

import os

import tomlkit


class Settings:
    # In a future release will add ability to specify the path when running app
    def __init__(self, file_name="soutools/settings/settings.toml"):
        self.file_name = file_name
        self.settings = None
        self.load_settings()

    def load_settings(self):
        home_dir = os.path.expanduser("~")
        settings_path = os.path.join(home_dir, self.file_name)

        if not os.path.exists(settings_path):
            # Create path and copy default settings.toml into it
            temp = os.path.split(settings_path)
            os.makedirs(temp[0])
            default_settings = tomlkit.loads(toml_template)
            try:
                with open(settings_path, "w") as toml_data:
                    toml_data.write(tomlkit.dumps(default_settings))
            except FileNotFoundError:
                raise ValueError(
                    f"Wasn't able to create default config file at location {settings_path}"
                )
        try:
            with open(settings_path, "r") as file:
                self.settings = tomlkit.loads(file.read())
        except tomlkit.exceptions.TOMLKitError as e:
            raise ValueError(f"Error decoding TOML file: {str(e)}")

    def get_value(self, key):
        if self.settings is None:
            raise ValueError(
                "Settings have not been loaded. Call load_settings() first."
            )

        keys = key.split(".")
        value = self.settings
        for key in keys:
            if isinstance(value, dict) and key in value:
                value = value[key]
            else:
                return None

        return value


# This is used to create a default template for the settings.toml file
toml_template = """# settings.toml

default_org_id = ''
default_org_name = ''

default_net_id = ''
default_net_name = ''

# Section 3.1 settings
# Generate a report of all sites of type wireless
[wireless_networks]
output_file = 'soutools/input/wireless.txt'

# Section 3.2 settings
# Generate a report of all sites with a particular SSID
[networks_with_ssid]
ssid = 'yourSSID'
input_file = 'soutools/input/wireless.txt'
output_file = 'soutools/output/myssid.txt'

# Section 3.3 settings
#Update radius servers for a particular SSID
[radius]
input_file = 'soutools/output/myssid.txt'

#------------------------------------------------------------------------------------#
# For both radius and accounting servers you can use any of the following formats:
# First format looks like this... (where the values are seperated by a semicolon)
#   servers = ['radius1.com;1812;firstSecret','radius2.com;1912;secondSecret']
#   ports = ''
#   secret = ''
# if using this format leave port and secret set as empty strings.
#
# The second format is for when the port and secret are the same for either
# the radius or accounting servers.
# Second scenario would like this...
#   servers = ['radius1.com','radius2.com']
#   port = 1812
#   secret = 'mysecret'
#------------------------------------------------------------------------------------#

[radius_servers]
servers = [
'radius1.company.com;1812;firstSecret',
'radius2.company.com;1912;secondSecret',
'radius3.company.com;2012;thirdSecret']
port = ''
secret = ''

[radius_accounting]
servers = [
'radius1.company.com',
'radius2.company.com',
'radius3.company.com']
port = 1813
secret = 'mySecret'

# Creating group policies
[group_policies]
source_network_id = ''
source_policy_name = ''
destinations_file = 'soutools/input/mysites.txt'

# Overall settings
[logging]
file_log_level = 'info'
file_log_path = 'soutools/output/logs/'

# Spefic to the meraki library
[meraki]
api_key = false
api_key_environment_variable = 'MERAKI_DASHBOARD_API_KEY'
suppress_logging = true
print_to_console = false
output_log = true
log_file_prefix = 'meraki_api_'
log_path = 'sou_meraki/output/logs'
"""



# controller.py

"""Controlls the main logic of the program"""

import logging
import sys

from soutools import helpers, menu, model

logger = logging.getLogger(__name__)
dashboard = model.MerakiModel()
org_id = helpers.get_settings.get_value("default_org_id")
org_name = helpers.get_settings.get_value("default_org_name")
net_id = helpers.get_settings.get_value("default_net_id")
net_name = helpers.get_settings.get_value("default_net_name")


def select_organization():
    global org_id, org_name
    logger.debug('The "select_organization" function called')
    org_id, org_name = dashboard.select_organization()
    logger.debug(f'The selected organization ID is "{org_id}"')
    logger.info(f'The selected organization name is "{org_name}"')


def view_organization():
    logger.debug('The "view_organization" function called')
    logger.debug("Checking if organization name is set")
    if not org_name:
        print("\nYou have not selected an organization yet\n")
        logger.debug("Notified user that organization name is not set")
    else:
        highlight_orgname = helpers.colorme(org_name, "blue")
        highlight_orgid = helpers.colorme(org_id, "blue")
        print(
            f'\nThe "{highlight_orgname}" organization with ID'
            f" {highlight_orgid} is currently selected\n"
        )
        logger.debug(f"Notified user that organization name is set to {org_name}")
    input("Press [ENTER] to continue...")


def select_network():
    global net_id, net_name
    logger.debug('The "select_network" function called')
    # Make sure org has been defined as network depends on it
    if not org_id:
        print("Please choose an organization first")
        input("Press [ENTER] to continue...")
    else:
        net_id, net_name = dashboard.select_network(org_id)
        if net_id:
            logger.debug(f'The selected network ID is "{net_id}"')
            logger.info(f'The selected network name is "{net_name}"')
        else:
            input("Press [ENTER] to continue...")


def view_network():
    logger.debug('The "view_network" function called')
    logger.debug("Checking if network name is set")
    if not net_name:
        print("\nYou have not selected a network yet\n")
        logger.debug("Notified user that network name is not set")
    else:
        highlight_netname = helpers.colorme(net_name, "blue")
        highlight_netid = helpers.colorme(net_id, "blue")
        print(
            f'\nThe "{highlight_netname}" network with ID'
            f" {highlight_netid} is currently selected\n"
        )
        logger.debug(f"Notified user that network name is set to {net_name}")
    input("Press [ENTER] to continue...")


def policy_options():
    menu_title = helpers.menu_title
    policy_menu = menu.Menu(
        menu_title,
        [
            ("Check a site for existing group policies", policy_report),
            ("View a particular group policy for a site", policy_view),
            ("Create a group policy", policy_create),
            ("Delete a particular group policy", policy_delete),
            ("Return to the main menu", main_menu),
            ("Exit", quit),
        ],
    )
    while True:
        policy_menu.get_input()


def policy_report():
    logger.debug('The "policy_report" function called')
    # Requires the network ID be set
    if not net_id:
        print("\nYou have not selected a network yet\n")
        logger.debug("Notified user that network name is not set")
    else:
        dashboard.get_group_policies(net_id)
        print()
    input("Press [ENTER] to continue...")


def policy_view():
    logger.debug('The "policy_view" function called')
    # Requires the network ID be set
    if not net_id:
        print("\nYou have not selected a network yet\n")
        logger.debug("Notified user that network name is not set")
    else:
        result = dashboard.view_group_policy(net_id)
        if not result:
            print("Specified policy was not found")
    input("\nPress [ENTER] to continue...")


def policy_create():
    logger.debug('The "policy_create" function called')
    warning = helpers.colorme("WARNING", "red")
    highlight_orgname = helpers.colorme(org_name, "blue")
    print(
        f"{warning} - You are about to make changes to the {highlight_orgname} organizaiton"
    )
    make_changes = input("Are you sure you want to continue [Y/N] ").upper()
    if "Y" in make_changes:
        print()
        dashboard.create_group_policy()
        input("Press [ENTER] to continue...")
    else:
        policy_options()


def policy_delete():
    logger.debug('The "policy_delete" function called')
    # Requires the network ID be set
    if not net_id:
        print("\nYou have not selected a network yet\n")
        logger.debug("Notified user that network name is not set")
        input("Press [ENTER] to continue...")
    else:
        warning = helpers.colorme("WARNING", "red")
        highlight_orgname = helpers.colorme(org_name, "blue")
        print(
            f"{warning} - You are about to make changes to the {highlight_orgname} organizaiton"
        )
        make_changes = input("Are you sure you want to continue [Y/N] ").upper()
        if "Y" in make_changes:
            print()
            result = dashboard.delete_group_policy(net_id)
            if not result:
                warning = helpers.colorme("WARNING", "red")
                print(f"{warning} Failed to delete the specified policy.")
                print("Please confirm that the name is correct and try again.")
            input("Press [ENTER] to continue...")
        else:
            policy_options()


def wireless_options():
    menu_title = helpers.menu_title
    wireless_menu = menu.Menu(
        menu_title,
        [
            ("Generate a report of all sites of type wireless", wireless_report),
            (
                "Generate a report of all sites with a particular SSID",
                wireless_report_with_ssid_number,
            ),
            ("Update radius servers for a particular SSID", update_radius_settings),
            ("Enable/Disable all sites with a particular SSID", enable_disable_ssid),
            ("Return to the main menu", main_menu),
            ("Exit", quit),
        ],
    )
    while True:
        wireless_menu.get_input()


def wireless_report():
    logger.debug('The "wireless_report" function called')
    if org_id:
        networks = dashboard.get_wireless_networks(org_id)
    else:
        print("No organization selected, please select organization first\n")
    path = helpers.get_settings.get_value("wireless_networks.output_file")
    helpers.writelines_to_file(path, networks)
    input("Press [ENTER] to continue...")


def wireless_report_with_ssid_number():
    logger.debug('The "wireless_with_ssid_x_report" function called')
    dashboard.find_ssid_number()
    input("Press [ENTER] to continue...")


def update_radius_settings():
    make_changes = "n"
    logger.debug('The "update_radius_settings" function called')
    warning = helpers.colorme("WARNING", "red")
    highlight_orgname = helpers.colorme(org_name, "blue")
    print(
        f"{warning} - You are about to make changes to the {highlight_orgname} organizaiton"
    )
    make_changes = input("Are you sure you want to continue [Y/N] ").upper()
    if "Y" in make_changes:
        print()
        dashboard.update_radius_servers()
        input("Press [ENTER] to continue...")
    else:
        wireless_options()


def enable_disable_ssid():
    ssid = helpers.get_settings.get_value("networks_with_ssid.ssid")
    menu_title = helpers.menu_title
    main_menu = menu.Menu(
        menu_title,
        [
            (f'Enable ssid "{ssid}" in the "{org_name}" org', enable_sites_with_ssid),
            (f'Disable ssid "{ssid}" in the "{org_name}" org', disable_sites_with_ssid),
            ("Return to wireless options", wireless_options),
            ("Exit", quit),
        ],
    )
    while True:
        main_menu.get_input()


def enable_sites_with_ssid():
    make_changes = "n"
    logger.debug('The "enable_sites_with_ssid" function called')
    warning = helpers.colorme("WARNING", "red")
    highlight_orgname = helpers.colorme(org_name, "blue")
    print(
        f"{warning} - You are about to make changes to the {highlight_orgname} organizaiton"
    )
    make_changes = input("Are you sure you want to continue [Y/N] ").upper()
    if "Y" in make_changes:
        print()
        dashboard.enable_disable_paticular_ssid(True)
        input("Press [ENTER] to continue...")
    else:
        enable_disable_ssid()


def disable_sites_with_ssid():
    make_changes = "n"
    logger.debug('The "enable_sites_with_ssid" function called')
    warning = helpers.colorme("WARNING", "red")
    highlight_orgname = helpers.colorme(org_name, "blue")
    print(
        f"{warning} - You are about to make changes to the {highlight_orgname} organizaiton"
    )
    make_changes = input("Are you sure you want to continue [Y/N] ").upper()
    if "Y" in make_changes:
        print()
        dashboard.enable_disable_paticular_ssid(False)
        input("Press [ENTER] to continue...")
    else:
        enable_disable_ssid()


def quit():
    helpers.clear_screen()
    sys.exit()


def main_menu():
    menu_title = helpers.menu_title
    main_menu = menu.Menu(
        menu_title,
        [
            ("Select an organization", select_organization),
            ("View the selected organization", view_organization),
            ("Select a network", select_network),
            ("View the selected network", view_network),
            ("Group Policy options", policy_options),
            ("Wireless options", wireless_options),
            ("Exit", quit),
        ],
    )
    while True:
        main_menu.get_input()



import os
import pytest
import tomlkit

from soutools.settings import Settings


@pytest.fixture
def settings():
    return Settings("soutools/settings/settings.toml")


def test_load_settings(settings):
    # Load the settings
    settings.load_settings()

    # Assert that the settings are loaded correctly
    assert settings.get_value('default_org_id') == '123'


def test_get_value(settings):
    # Load the settings
    settings.settings = tomlkit.loads('key = "value"')

    # Assert that the value is retrieved correctly
    assert settings.get_value("key") == "value"


def test_get_value_nested(settings):
    # Load the settings
    settings.settings = tomlkit.loads('key1 = { key2 = "value" }')

    # Assert that the nested value is retrieved correctly
    assert settings.get_value("key1.key2") == "value"


def test_get_value_settings_not_loaded(settings):
    # Assert that a ValueError is raised when trying to get a value without loading the settings first
    with pytest.raises(ValueError):
        settings.get_value("key")


def test_load_settings_file_not_found(tmpdir):
    # Assert that a FileNotFoundError is raised when the settings file does not exist
    settings_file = tmpdir.join("settings.toml")
    settings = Settings(settings_file)
    with pytest.raises(FileNotFoundError):
        settings.load_settings()


def test_load_settings_invalid_toml(tmpdir):
    # Create a temporary settings file with invalid TOML syntax
    settings_file = tmpdir.join("settings1.toml")
    settings_file.write('key = value')
    settings = Settings(settings_file)
    # Assert that a ValueError is raised when loading the settings
    with pytest.raises(ValueError):
        settings.load_settings()


# helpers.py

"""This module is used to assists the other modules with functions like writting to file"""

import logging
import os
import platform
import sys

from soutools import settings

logger = logging.getLogger(__name__)
get_settings = settings.Settings()
home_dir = os.path.expanduser("~")


def setup_folder_structure(path):
    logger.debug('The "setup_folder_structure" function called')
    logger.debug(f"Checking the path {path} for existance and creating if not exists")
    temp_path = os.path.split(path)
    path = temp_path[0]
    if not os.path.exists(path):
        logger.debug(f"making directory structure for {path}")
        os.makedirs(path)


def writelines_to_file(path, text):
    logger.debug('The "writelines_to_file" function called')
    # Find users home directory and join with settings location
    location = os.path.join(home_dir, path)
    logger.debug(f"The path for writelines_to_file is {location}")
    setup_folder_structure(location)

    # Write text to given path
    try:
        with open(location, "w") as file_data:
            file_data.writelines(text)
    except FileNotFoundError:
        logger.error(f"Could not open {location}")
        sys.exit()


def validate_integer_in_range(end_range):
    """Prompts user to select a number within a defined range

    Args:
        end_range (int): The upper bound number that can be selected

    Returns:
        int: The selected number
    """

    while True:
        try:
            selected = int(input("\n  Select an option >> "))
            assert selected in range(1, end_range + 1)
        except ValueError:
            print("\tThat is not an integer!\n")
        except AssertionError:
            print(f"\n\tYou must enter a number between 1 and {end_range}")
        else:
            break
    return selected - 1


def get_networks_list(path):
    filepath = os.path.join(home_dir, path)
    networks = []
    try:
        with open(filepath, "r") as netdata:
            for line in netdata:
                if not line.isspace():
                    networks.append(line.strip())
        return networks
    except FileNotFoundError:
        logger.error(f"Could not open {path}")
        sys.exit()


def format_radius():
    # Retrieve settings
    radius = get_settings.get_value("radius_servers")
    accounting = get_settings.get_value("radius_accounting")
    radius_port = radius["port"]
    radius_secret = radius["secret"]
    accounting_port = accounting["port"]
    accounting_secret = accounting["secret"]
    # Initialize payloads
    radius_payload = []
    accounting_payload = []
    # Format Radius settings
    if radius_port and radius_secret:
        for server in radius["servers"]:
            temp = {"host": server, "port": radius_port, "secret": radius_secret}
            radius_payload.append(temp)
    else:
        for server in radius["servers"]:
            server_info = server.split(";")
            temp = {
                "host": server_info[0],
                "port": int(server_info[1]),
                "secret": server_info[2],
            }
            radius_payload.append(temp)
    # Format Accounting settings
    if accounting_port and accounting_secret:
        for server in accounting["servers"]:
            temp = {
                "host": server,
                "port": accounting_port,
                "secret": accounting_secret,
            }
            accounting_payload.append(temp)
    else:
        for server in accounting["servers"]:
            server_info = server.split(";")
            temp = {
                "host": server_info[0],
                "port": int(server_info[1]),
                "secret": server_info[2],
            }
            accounting_payload.append(temp)
    # Return the formatted payloads
    return radius_payload, accounting_payload


def progress_bar(progress, total, width=40):
    char = chr(9632)
    logger.debug(f"progress is {progress}")
    logger.debug(f"total is {total}")
    if progress >= total:
        logger.debug("finished has been set to true")
        fill_char = colorme(char, "green")
    else:
        fill_char = colorme(char, "red")
    completed = int(width * (progress / total))
    bar = "Progress: [" + fill_char * completed + "-" * (width - completed) + "] "
    percent_done = round(progress / total * 100, 1)
    bar += str(percent_done) + "% " + str(progress) + "/" + str(total)
    return bar


def colorme(msg, color):
    if color == "red":
        wrapper = "\033[91m"
    elif color == "blue":
        wrapper = "\033[94m"
    elif color == "green":
        wrapper = "\033[92m"
    else:
        # Defaults to white if invalid color is given
        wrapper = "\033[47m"
    return wrapper + msg + "\033[0m"


def clear_screen():
    if platform.system().lower() == "windows":
        cmd = "cls"
    else:
        cmd = "clear"
    os.system(cmd)


# https://patorjk.com/software/taag/ using formatting "standard"

menu_title = colorme(
    r"""
   ____        _   _ _____           _
  / ___|  ___ | | | |_   _|__   ___ | |___
  \___ \ / _ \| | | | | |/ _ \ / _ \| / __|
   ___) | (_) | |_| | | | (_) | (_) | \__ \
  |____/ \___/ \___/  |_|\___/ \___/|_|___/
""",
    "red",
)



# model.py

"""This module handles the interface between the controller and the Meraki Dashboard"""

import json
import logging
import os
import sys

import meraki

from soutools import helpers

logger = logging.getLogger(__name__)
logger.info("Loading settings for the dashboard instance")


class MerakiModel:
    def __init__(self) -> None:
        self.logger = logging.getLogger(__name__)
        logger.info("Instantiated an instance of the model.MerakiModel class")
        if helpers.get_settings.get_value("meraki.api_key"):
            api_key = helpers.get_settings.get_value("meraki.api_key")
        elif (
            helpers.get_settings.get_value("meraki.api_key_environment_variable")
            in os.environ
        ):
            api_key = os.environ.get(
                helpers.get_settings.get_value("meraki.api_key_environment_variable")
            )
        else:
            logger.error("API Key not found")
            sys.exit("ERROR: API Key not found")

        # Instantiate an instance of the Meraki dashboard
        self.dashboard = meraki.DashboardAPI(
            api_key=api_key,
            output_log=helpers.get_settings.get_value("meraki.output_log"),
            print_console=helpers.get_settings.get_value("meraki.print_to_console"),
            suppress_logging=helpers.get_settings.get_value("meraki.suppress_logging"),
            log_file_prefix=helpers.get_settings.get_value("meraki.log_file_prefix"),
            log_path=helpers.get_settings.get_value("meraki.log_path"),
        )

    def select_organization(self):
        """Lists all organizations and prompts the user to select one

        Args:
            dashboard (obj): The Meraki dashboard instance

        Returns:
            str: the selected organization ID
            str: the selected organization name
        """
        logger.debug('The "select_organization" function called from the model')
        organizations = None
        try:
            logger.debug("Trying to get organizations from the Meraki dashboard")
            organizations = self.dashboard.organizations.getOrganizations()
            logger.debug(f"organizations is equal to {organizations}")
        except meraki.exceptions.APIError:
            sys.exit("Check network settings and verify API Key")
        organizations.sort(key=lambda x: x["name"])
        print("  Organizations to choose from:\n")
        for line_num, organization in enumerate(organizations, start=1):
            name = helpers.colorme(f"{line_num} - {organization['name']}", "green")
            print(f"    {name}")
        selected = helpers.validate_integer_in_range(len(organizations))
        return (
            organizations[int(selected)]["id"],
            organizations[int(selected)]["name"],
        )

    def select_network(self, org, lines_to_display=25):
        """Lists the organization networks and prompts user to select one

        Args:
            dashboard (obj): The Meraki dashboard instance
            org (str): The selected organization ID
            lines_to_display (int): The number of lines before pausing

        Returns:
            list: the selected network ID and network name
        """

        network_list = []
        try:
            logger.debug("Trying to get networks from the Meraki dashboard")
            networks = self.dashboard.organizations.getOrganizationNetworks(org)
            while not network_list:
                search_name = input(
                    "Enter a name to search for or leave blank for all networks: "
                )
                if search_name:
                    for network in networks:
                        if search_name.lower() in network["name"].lower():
                            network_list.append(network)
                else:
                    network_list = networks
                if network_list:
                    network_list.sort(key=lambda x: x["name"])
                    print("\nNetworks:")
                    for line_num, net in enumerate(network_list, start=1):
                        net_name = net["name"]
                        print(f"{line_num} - {net_name}")
                        if line_num % lines_to_display == 0:
                            user_response = input(
                                "\nPress Enter to continue, or q + Enter to quit search: "
                            )
                            if "q" in user_response:
                                break
                else:
                    print(f"No networks found matching {search_name}")

            selected = helpers.validate_integer_in_range(len(network_list))
            return [
                network_list[int(selected)]["id"],
                network_list[int(selected)]["name"],
            ]

        except Exception as err:
            logger.debug(
                f"Error while trying to get organization networks for org ID {org}"
            )
            logger.debug(err.args)
            print(
                helpers.colorme(
                    "Error retieving networks, verify org ID is set properly", "red"
                )
            )
            return ("", "")

    def get_group_policies(self, netid):
        logger.debug(
            f'The "get_group_policies" function called with network ID {netid}'
        )
        group_policies = []
        group_policies = self.dashboard.networks.getNetworkGroupPolicies(netid)
        if not group_policies:
            logger.info(f"No group policies found for network ID {netid}")
            net_id = helpers.colorme(netid, "blue")
            print(f"  No group policies found for network ID {net_id}")
        for gp in group_policies:
            logger.info(f"Found group policy with name \"{gp['name']}\"")
            gp_name = helpers.colorme(gp["name"], "blue")
            print(f"  Found Policy Name: {gp_name}")

    def view_group_policy(self, netid):
        logger.debug(f'The "view_group_policy" function called with net_id {netid}')
        policy_name = input("Please enter the name of the group policy to view: ")
        group_policies = self.dashboard.networks.getNetworkGroupPolicies(netid)
        for gp in group_policies:
            if policy_name == gp["name"]:
                print(json.dumps(gp, indent=2))
                return True
        logger.info(f'The specified group policy "{policy_name}" was not found')
        return False

    def create_group_policy(self):
        logger.debug('The "create_group_policy" function called')
        group_policy = []
        policy_name = policy_source = destinations = ""
        # Load the values needed to complete this operation or exit
        if helpers.get_settings.get_value("group_policies.source_policy_name"):
            policy_name = helpers.get_settings.get_value(
                "group_policies.source_policy_name"
            )
        if helpers.get_settings.get_value("group_policies.source_network_id"):
            policy_source = helpers.get_settings.get_value(
                "group_policies.source_network_id"
            )
        if helpers.get_settings.get_value("group_policies.destinations_file"):
            destinations = helpers.get_settings.get_value(
                "group_policies.destinations_file"
            )
        if not policy_name or not policy_source or not destinations:
            logger.info("Missing some values that are needed for create operation")
            logger.info("Please confirm group_policies section in settings.toml file")
            print(helpers.colorme("Error: Missing values needed for operation", "red"))
            sys.exit("Please fill out the group_policies section in the settings file")
        # Pull source policy from specified source network
        group_policies = self.dashboard.networks.getNetworkGroupPolicies(policy_source)
        for policy in group_policies:
            if policy_name == policy["name"]:
                group_policy = policy
        if not group_policy:
            logger.info(f"Specified source policy name {policy_name} was not found.")
            logger.info("Please ensure that the source policy name is correct")
            print(helpers.colorme("Error: Cannot continue", "red"))
            sys.exit("Please ensure that the source policy name is correct")
        # Load the destination networks from the specified file
        networks = helpers.get_networks_list(destinations)
        # Prepare the progress bar
        progress = 0
        total = len(networks)
        # Everything is ready to go. Write the group policy to each requested network
        for network in networks:
            bar = helpers.progress_bar(progress, total)
            print(bar, end="", flush=True)
            network_id = network.split(",")[0]
            network_name = network.split(",")[1]
            if not (network_id.startswith("L_") or network_id.startswith("N_")):
                logger.info("Check the destination input file for errors")
                logger.info(f"The network ID {network_id} is incorrect")
                print(helpers.colorme("Error: Cannot continue", "red"))
                sys.exit("Found an invalid network ID, check destinations file")
            try:
                logger.debug(f"Creating group policy for network {network_name}")
                self.dashboard.networks.createNetworkGroupPolicy(
                    network_id,
                    policy_name,
                    scheduling=group_policy["scheduling"],
                    bandwidth=group_policy["bandwidth"],
                    firewallAndTrafficShaping=group_policy["firewallAndTrafficShaping"],
                    contentFiltering=group_policy["contentFiltering"],
                    # Wireless only settings for the next three
                    splashAuthSettings=group_policy["splashAuthSettings"],
                    vlanTagging=group_policy["vlanTagging"],
                    bonjourForwarding=group_policy["bonjourForwarding"],
                )
                logger.info(
                    f"Group policy {policy_name} created for network {network_name}"
                )
            except Exception as api_error:
                logger.info(f"Failed to write group policy for {network_name}")
                logger.info(api_error)
            progress += 1
            print("\b" * len(bar), end="", flush=True)
        bar = helpers.progress_bar(progress, total)
        print(bar, end="", flush=True)
        print("\n")

    def delete_group_policy(self, netid):
        logger.debug(f'The "delete_group_policy" function called with net_id {netid}')
        policy_name = input("Please enter the name of the group policy to delete: ")
        group_policies = self.dashboard.networks.getNetworkGroupPolicies(netid)
        for gp in group_policies:
            if policy_name == gp["name"]:
                self.dashboard.networks.deleteNetworkGroupPolicy(
                    netid, gp["groupPolicyId"]
                )
                logger.info(
                    f"Successfully deleted group policy with name {policy_name}"
                )
                return True
        logger.info(f"Failed to delete group policy with name {policy_name}")
        logger.info("Please confirm that the name is correct and try again.")
        return False

    def get_wireless_networks(self, organization_id):
        logger.debug(
            f'The "get_wireless_networks" function called with org_id {organization_id}'
        )
        networks = self.dashboard.organizations.getOrganizationNetworks(
            organization_id, total_pages="all"
        )
        total = 0
        wireless = 0
        no_wireless = 0
        wireless_networks = []
        for network in networks:
            if "wireless" in network["productTypes"]:
                wireless_networks.append(network["id"] + "," + network["name"] + "\n")
                logger.info(
                    f'network {network["name"]}, \
                    was identified as having some type of wireless'
                )
                wireless += 1
                total += 1
            else:
                logger.info(
                    f"network {network['name']}, \
                    was identified as not having some type of wireless"
                )
                no_wireless += 1
        logger.info(
            f"INFO: There are {wireless} sites with some type of wireless device"
        )
        logger.info(f"INFO: There were {no_wireless} sites with no wireless")
        return wireless_networks

    def find_ssid_number(self):
        logger.debug('The "wheres_ssid" function called')
        search_ssid = helpers.get_settings.get_value("networks_with_ssid.ssid")
        input_file = helpers.get_settings.get_value("networks_with_ssid.input_file")
        networks = helpers.get_networks_list(input_file)
        path = helpers.get_settings.get_value("networks_with_ssid.output_file")
        logger.debug(f'Searching for ssid "{search_ssid}"')
        logger.debug(f'Searching for networks found in "{input_file}"')
        logger.debug(f'Writting to path "{path}"')
        progress = found_count = not_found_count = 0
        total = len(networks)
        logger.info(f"The total number of networks is {total}")
        ssid_sites = []
        print()
        for network in networks:
            bar = helpers.progress_bar(progress, total)
            print(bar, end="", flush=True)
            site = network.split(",")
            site_id = site[0]
            site_name = site[1]
            logger.debug(
                f"Calling Meraki dashboard to pull all SSIDs for network {site_id}"
            )
            ssids = self.dashboard.wireless.getNetworkWirelessSsids(site_id)
            found_ssid = False
            for ssid in ssids:
                if ssid["name"] == search_ssid:
                    found_ssid = True
                    newline = f'{site_id},{site_name},{ssid["name"]},\
                    {ssid["number"]},{ssid["enabled"]}\n'
                    ssid_sites.append(newline)
                    logger.info(f'Found SSID {search_ssid} for "{site_name}"')
                    found_count += 1
                    progress += 1
            if not found_ssid:
                logger.info(f'SSID {search_ssid} was not found for "{site_name}"')
                not_found_count += 1
                progress += 1
            print("\b" * len(bar), end="", flush=True)
        bar = helpers.progress_bar(progress, total)
        print(bar, end="", flush=True)
        print("\n")
        helpers.writelines_to_file(path, ssid_sites)
        logger.info(f"Sites with SSID {search_ssid}, found {found_count}")
        logger.info(f"Sites without SSID {search_ssid}, found {not_found_count}")

    def update_radius_servers(self):
        logger.debug('The "update_radius_servers" function called')
        input_file = helpers.get_settings.get_value("radius.input_file")
        networks = helpers.get_networks_list(input_file)
        radius, accounting = helpers.format_radius()
        logger.debug(f'Searching for networks "{networks}"')
        logger.debug(f'Searching for radius servers "{radius}"')
        logger.debug(f'Searching for accounting servers "{accounting}"')
        progress = 0
        total = len(networks)
        for network in networks:
            bar = helpers.progress_bar(progress, total)
            print(bar, end="", flush=True)
            site = network.split(",")
            net_id = site[0]
            net_name = site[1]
            ssid_num = site[3]
            logger.info(
                f"Updating radius settings for {net_name} and SSID number {ssid_num}"
            )
            logger.debug(
                f"Calling Meraki dashboard with"
                f"{net_id} {ssid_num} {radius} {accounting}"
            )
            try:
                self.dashboard.wireless.updateNetworkWirelessSsid(
                    networkId=net_id,
                    number=ssid_num,
                    radiusServers=radius,
                    radiusAccountingServers=accounting,
                )
            except Exception as api_error:
                logger.info(f"Failed to update SSID for {net_name}")
                logger.debug(api_error)
            progress += 1
            print("\b" * len(bar), end="", flush=True)
        logger.info(f"Updated radius settings on {progress} SSIDs")
        bar = helpers.progress_bar(progress, total)
        print(bar, end="", flush=True)
        print("\n")

    def enable_disable_paticular_ssid(self, ssid_enabled):
        logger.debug('The "enable_disable_paticular_ssid" function called')
        status = "enabled" if ssid_enabled else "disabled"
        input_file = helpers.get_settings.get_value("ssid_enabled_status.input_file")
        networks = helpers.get_networks_list(input_file)
        progress = 0
        total = len(networks)
        print()
        for network in networks:
            bar = helpers.progress_bar(progress, total)
            print(bar, end="", flush=True)
            site = network.split(",")
            site_id = site[0]
            site_name = site[1]
            ssid_name = site[2]
            ssid_num = site[3]
            logger.info(f"Changing status for {site_name} ssid {ssid_name} to {status}")
            try:
                self.dashboard.wireless.updateNetworkWirelessSsid(
                    networkId=site_id, number=ssid_num, enabled=ssid_enabled
                )
            except Exception as api_error:
                logger.info(f"Failed to update SSID for {site_name}")
                logger.debug(api_error)
            progress += 1
            print("\b" * len(bar), end="", flush=True)
        bar = helpers.progress_bar(progress, total)
        print(bar, end="", flush=True)
        print("\n")
